// USB-PD Solder
// for ATtiny44, 8MHz
// [23/03/22] Created by rev_c1815@yahoo.co.jp


#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* for sei(), timer0 */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include <stdint.h>
#include "YSwitchProc.h"

volatile uint8_t g_swState;

#define HEATER_RESISTANCE 3.125 // mohm
// 3.125x1024x1024/100 = 32768 (0x8000)

struct sPreference{
	uint8_t curPowerLevel;
};
struct sPreference g_preference;
#define EEP_START_ADDR 0x10



void LoadEEPSetting(struct sPreference* pSetting)
{
	uint8_t signature = eeprom_read_byte((uint8_t*)EEP_START_ADDR);
	if(signature == 0x55){
		eeprom_read_block(pSetting, (uint8_t*)EEP_START_ADDR+1, sizeof(struct sPreference));
	}
	if(pSetting->curPowerLevel > 6) pSetting->curPowerLevel = 0;
}
void SaveEEPSetting(const struct sPreference* pSetting)
{
	// signature
	eeprom_write_byte((uint8_t*)EEP_START_ADDR, 0x55);
	eeprom_write_block(pSetting, (uint8_t*)EEP_START_ADDR+1, sizeof(struct sPreference));
}

void LoadEEPSetting(struct sPreference* pSetting);
void SaveEEPSetting(const struct sPreference* pSetting);
extern struct sPreference g_preference;

volatile uint8_t g_100msFlag;

// ADC
ISR(ADC_vect)
{
}

// Timer 1 (100ms)
ISR(TIM1_COMPA_vect)
//ISR(TIM1_OVF_vect)
{
	uint8_t sw_pin = ((PINA & 0x1) << 2) | (PINB & 0x3);
	GetSwitchState(sw_pin, (uint8_t*)&g_swState);
	g_100msFlag = 1;
}


uint16_t GetADCResult(uint8_t ch)
{
	ADMUX = ch;
	_delay_us(100);
	ADCSRA |= 0b01000000; // Start
	while(ADCSRA & 0x40);
	return ADC;
}

static void Init(void)
{
	DDRB  = 0b00000100; // PB2: output (OC0A = PWM)
	PORTB = 0b00000011; // SW1,2 pull-up
	
	// PA7,3,1: analog input
	DDRA  = 0b01110100;
	PORTA = 0b00000001; // SW3 Pull-up
	//DIDR0 = 0b10001010;
	
	
	// Timer 0 (PWM) 8M/256 = 31 kHz
	OCR0A = 0;
	
	TCCR0A = 0b10000011; // PWM mode, OC0A enabled
	TCCR0B = 0b00000001; // TOP = 256, 8MHz
	
	
	// Timer 1 (100ms)
	OCR1A  = 800;
	TCCR1A = 0;
	TCCR1B = 0b00001101; // 8M/1024 = 8kHz
	TIMSK1 |= 0b00000010; // OCIE1A = 1
//	TIMSK1 |= 0b00000001; // TOIE1A = 1
	
	// ADC
//	ADMUX  = 0b10000111; // REF = 1.1V, MUX=ADC7
//	ADCSRA = 0b10001110; // ADPS = 1/64 (125 kHz), Interrupt Enable
	ADCSRA = 0b10000101; // ADPS = 1/32 (255 kHz), Interrupt Disable
	ADCSRB = 0x00; // Continuous conversion
	//ADCSRA |= 0b01000000; // Start
	DIDR0 = 0x80;
}

#define LED_MASK 0b10101011
int main(void)
{
	Init();
	
	const uint8_t ledList[3] = {0b01000000, 0b01010000, 0b01010100};
	const uint8_t pwmValue[3] = {256*0.938, 256*0.347, 256*0.424};
	
	sei();
	
	uint8_t ledBlinkFlag = 0, errorFlag = 0;
	uint8_t heaterEnable = 0;
	
	// check CC pin voltage
	uint16_t ccVoltage1  = GetADCResult(3);
	uint16_t ccVoltage2  = GetADCResult(1);
	
	while(1)
	{
		uint8_t swState = g_swState;
		g_swState = 0;
		uint8_t curLevel = g_preference.curPowerLevel;
		if(swState == SW_SW1) // +
		{
			if(curLevel < 2) curLevel ++;
		}
		else if(swState == SW_SW2) // -
		{
			if(curLevel > 0) curLevel --;
		}
		else if(swState == SW_SW3) // ON/OFF
		{
			heaterEnable ^= 1;
		}
		
		
		// Check source voltage and current
		errorFlag = 0;
		if(curLevel == 0)
		{
			// Get ADC Result (VREF = 3.3V)
			uint16_t ccVoltage1  = GetADCResult(3);
			uint16_t ccVoltage2  = GetADCResult(1);
			
			// CC < 0.61V? (Default current)
			if(ccVoltage1 < (uint16_t)(0.61*1024/3.3) && 
				ccVoltage2 < (uint16_t)(0.61*1024/3.3))
			{
				errorFlag = 1;
			}
		}
		else
		{
			// Get ADC Result (VREF = 3.3V)
			uint16_t vbusVoltage = GetADCResult(7);
			
			// vbus < 8V ?
			if(vbusVoltage < (uint16_t)(8/11*1024/3.3))
			{
				errorFlag = 1;
			}
		}
		
		// Update PWM Value
		if(heaterEnable && (errorFlag == 0)){
			OCR0A = pwmValue[curLevel];
		}else{
			OCR0A = 0;
		}
		
		// VBUS Request Voltage
		if(curLevel == 0){
			PORTA |= 0x20; // CFG1 = 1 (5V)
		}else{
			PORTA &= ~0x20; // CFG1 = 0 (9V)
		}
		g_preference.curPowerLevel = curLevel;
		
		// Update LED
		if(g_100msFlag)
		{
			g_100msFlag = 0;
			ledBlinkFlag ++;
			
			PORTA = (PORTA & LED_MASK);
			if(heaterEnable == 0) // blink at 1/8 duty
			{
				if((ledBlinkFlag & 7) == 0) PORTA |= ledList[g_preference.curPowerLevel];
			}
			else
			{
				if((errorFlag == 0) || (ledBlinkFlag & 1)) PORTA |= ledList[g_preference.curPowerLevel];
			}
		}
		
		
	}
	
	return 0;
}

